/*

        Sky Screen Saver for Windows

				Definitions

*/

#define ScreenSaver

#include <windows.h>
#include <winbase.h>
#include <scrnsave.h>
#include <string.h>
#include <time.h>
#ifndef RC_INVOKED
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <math.h>
#include <sys/types.h>
#include <sys/timeb.h>
#endif

#define MoveTo(h, x, y)	MoveToEx((h), (x), (y), NULL)

#include "resource.h"

//      String codes in resource file

#define IDS_DAY_NAMES           16          	// Day name base

#define IDS_APPNAME				103				// Application name/title
#define IDS_GREEK_FONTNAME      162             // Name of font (Symbol) for Greek letters
#define IDS_DEEP_CODES			163				// Deep sky object code table

#define IDS_NORTH				154				// North
#define IDS_SOUTH				155				// South
#define IDS_EAST				156				// East
#define IDS_WEST				157				// West

#define IDS_FORMATS            200          	// Format base
#define Format(n)   rstring(IDS_FORMATS + (n))

//      Resource ID numbers

#define IDR_PLANETS			1700            // Planet resource codes (need 11)
#define IDR_SUN				1700
#define IDR_MERCURY			1701
#define IDR_VENUS			1702
#define IDR_EARTH			1703
#define IDR_MARS			1704
#define IDR_JUPITER			1705
#define IDR_SATURN			1706
#define IDR_URANUS			1707
#define IDR_NEPTUNE			1708
#define IDR_PLUTO			1709
#define IDR_ASTEROID		1710
#define IDR_COMET			1711

#define IDR_STARS			1751			// Star stampers
#define IDR_STAR1			1751
#define IDR_STAR2			1752
#define IDR_STAR3			1753
#define IDR_STAR4			1754
#define IDR_STAR5			1755
#define IDR_STAR6			1756
#define IDR_STAR7			1757
#define IDR_STAR8			1758
#define IDR_STAR9			1759

#define IDR_DEEP			1781			// Deep sky icons
#define IDR_DEEPEG			1781
#define	IDR_DEEPSG			1782
#define IDR_DEEPOC			1783
#define IDR_DEEPGC			1784
#define IDR_DEEPDN			1785
#define IDR_DEEPPN			1786
#define IDR_DEEPQS			1787
#define IDR_DEEPDS			1788
#define IDR_DEEPDK			1789
#define IDR_DEEPIG			1790

//	Definitions for screen saver unique objects

#define ID_BITMAP					100
#define ID_SHOWMOON					101
#define ID_STARTUP					102
#define ID_HELP                     110

#define PI 3.14159265358979323846  /* Assume not near black hole nor in
                                      Tennessee */

#define V       (void)

#define EOS     '\0'

//	Frequently used astronomical constants

#define J2000				2451545.0		// Julian day of J2000 epoch
#define JulianCentury		36525.0			// Days in Julian century
#define AstronomicalUnit	149597870.0		// Astronomical unit in kilometres
#define SunSMAX	 (AstronomicalUnit * 1.000001018) // Semi-major axis of Earth's orbit
#define EarthRad			6378.14			// Earth's equatorial radius, km (IAU 1976)
#define LunatBase			2423436.0		/* Base date for E. W. Brown's numbered
											   series of lunations (1923 January 16) */
#define SynMonth			29.53058868		// Synodic month (mean time from new Moon to new Moon)

//	Precession calculation modes

#define PrecAuto		0					// Precess if more then PrecYears from PrecEpoch
#define PrecAlways		1					// Always precess
#define PrecNever		2					// Never correct for precession
#define PrecEpoch		J2000				// All databases are epoch J2000.0
#define PrecYears		25					// Consider databases valid for this time around epoch

/*  Handy mathematical functions  */

#ifdef abs
#undef abs
#endif

/* As an indication of how carefully Microsoft tests software prior
   to shipment, and their deep committment to numerical applications,
   Visual C++ 1.5 (C 8.0) shipped with a library in which tan(x)
   returns the wrong sign for arguments between -90 and 90 degrees
   but only if the math coprocessor is not installed.  Can you believe
   this?  So, define tan(x) as sin(x) / cos(x) to work around this
   unbelievable example of lack of regression testing. */
   
//#define MICROSOFT_CAN_T_COUNT_WORTH_BEANS
#ifdef MICROSOFT_CAN_T_COUNT_WORTH_BEANS
#define tan(x)	(sin(x) / cos(x))
#endif // MICROSOFT_CAN_T_COUNT_WORTH_BEANS 

#define sgn(x) (((x) < 0) ? -1 : ((x) > 0 ? 1 : 0))       /* Extract sign */
#define abs(x) ((x) < 0 ? (-(x)) : (x))                   /* Absolute val */
#define fixangle(a) ((a) - 360.0 * (floor((a) / 360.0)))  /* Fix angle    */
#define fixangr(a)  ((a) - (PI*2) * (floor((a) / (PI*2))))/* Fix angle in radians*/
#define dtr(x) ((x) * (PI / 180.0))                       /* Degree->Radian */
#define rtd(x) ((x) / (PI / 180.0))                       /* Radian->Degree */

#define NeedToCalculate(qty, time)  ((abs((qty##Last) - (time)) > qty##Interval) ? \
                                     (qty##Last = (time), TRUE) : FALSE)
#define InvalidateCalculation(qty)      qty##Last = -1e10
#define ValidateCalculation(qty, time) qty##Last = (time)
#define CalculationInterval(x)  ((x) / (24.0 * 60 * 60))

#define RepositionTime	10		// Reposition image every RepositionTime minutes

//  Determine number of elements in an array

#define ELEMENTS(array) (sizeof(array)/sizeof((array)[0]))

//  Calculate the instaneous free space on the stack and report in a message box

#ifndef NDEBUG
#define ProbeStack() { extern int stackavail(void);						\
                       char s[40];										\
                       sprintf(s, "%d", stackavail());					\
                       MessageBox(hDlg, s, "Free stack", MB_OK); }
#endif                       

//  Assertion checking for Windows

#ifndef NDEBUG
#define assert(x) { if (!(x)) { char erm[128]; wsprintf((LPSTR) erm, \
        (LPSTR) "Assertion %s failed at line %d of file %s.", (LPCSTR) #x, __LINE__,(LPSTR) __FILE__); \
        FatalAppExit(0, (LPCSTR) erm); }}
#else
#define assert(x)
#endif

//	Log reaching a point in the code

#ifndef APSTUDIO_INVOKED
#ifndef NDEBUG
#define RX { char erm[80]; wsprintf((LPSTR) erm, \
        (LPSTR) "Line %d of %s.\r\n", __LINE__, (LPSTR) __FILE__); \
        OutputDebugString(erm); }
#else
#define RX
#endif
#endif

//	Make sure no debug output remains in production builds: use Honk(x) for debug MessageBeep(x)

#ifdef NDEBUG
#ifdef OutputDebugString
#undef OutputDebugString
#endif
#define OutputDebugString(x) DebugOutputInProductionBuild()
#define Honk(x) DebugOutputInProductionBuild()
#else
#define Honk(x)	MessageBeep(x)
#endif

extern char szString[128];		// Variable to load resource strings
#define hInst	 hMainInstance  // Redefinitions for screen saver lib compatibility
#define hWndMain hMainWindow
extern HINSTANCE hInst;			// Class instance pointer
extern HWND hWndMain;			// Main window pointer
extern HICON micon;				// Moon icon calculated for correct phase
extern HICON planetIcons[10];	// Planet icons
extern double faketime;         // Time for simulation / testing
extern double siteLat;          // Observing site latitude
extern double siteLon;          // Obeserving site longitude
extern char *tzName[2];			// Time zone names
extern int plutoPrecise;        // Precise position for Pluto required
extern double skyLimitMag;		// Sky limiting magnitude
extern double skyNameMag;		// Star name limiting magnitude
extern int skyShowName;			// Show star names in sky map ?
extern int skyShowBflam;		// Show Bayer/Flamsteed numbers in sky ?
extern double skyBflamMag;		// Bayer/Flamsteed magnitude cutoff
extern int skyShowDeep;			// Show deep sky objects in sky ?
extern double skyDeepMag;		// Deep sky magnitude cutoff
extern int precessionCalculation; // Precession calculation mode
extern int skyShowConstellations; // Show constellations in sky window ?
extern int skyShowConbounds;	// Show constellation boundaries in sky map ?
extern int skyShowConnames;		// Show constellation names in sky map ?
extern int skyAlignConnames;	// Align constellation names with map ?
extern int skyShowCoords;		// Show coordinate grid ?
extern int skyShowPlanets;		// Show solar system objects ?
extern int starQuality;			// Star quality rendering
extern int starCatalogue;		// Star catalogue selected
extern int showStarColours;		// Show star colours ?
extern int skyShowTiming;		// Show calculation time ?

struct planet {                 // Planet information entry
	double hlong;				// Heliocentric longitude
	double hlat;				// Heliocentric latitude
	double hrv;					// Heliocentric radius vector
    double ra;                  // Current right ascension
    double dec;                 // Current declination
    double dist;				// Distance from the Earth
    double mag;					// Approximate magnitude
    double lha;					// Local hour angle
    double alt;					// Altitude above (-below) horizon
    double az;					// Azimuth from South: West positive, East negative
};
extern struct planet planet_info[11];	// Calculated planetary information

#define PlanetIcon(n)	planetIcons[(n)]

#define WindowProc(x) LONG FAR PASCAL x(HWND hWnd, UINT Message, WPARAM wParam, LPARAM lParam)

extern WindowProc(SkyWndProc);

/* Dialogue service functions. */

#define DialogueProc(name) BOOL FAR PASCAL name(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)

extern DialogueProc(About);
extern DialogueProc(TimeZoneDlg);

// 	DLL function type definition

#define DLL

//	From SUNMOON.C

extern void upd_sun_moon(double jd, int realtime, int repaint);
extern void updMoonIcon(HDC hDC, double jd);

//	From PLANETP.C

extern void updatePlanet(double jd, int normal);
extern void planetSiteChanged(void);
extern void calcPlanet(double jd);

//	From PLANETS.C (in DLL)

extern void DLL planets(double jd, int which, struct planet FAR *planet_info);	/* Update planetary positions */

//	From SKY.C

extern void loadPlanetIcons(void);
extern void unloadPlanetIcons(void);
extern void updateSky(double jd, int force);

//  Calculation functions from ASTRO.C (in DLL)

extern double DLL jtime(struct tm FAR *t);
extern void DLL jyear(double td, long FAR *yy, int FAR *mm, int FAR *dd),
            DLL jhms(double j, int FAR *h, int FAR *m, int FAR *s);
extern double DLL gmst(double jd);
extern double DLL ucttoj(long year, int mon, int mday,
                     int hour, int min, int sec);
extern double DLL phase(
					double  pdate, double  FAR *pphase, double  FAR *mage, double  FAR *dist,
					double  FAR *angdia, double  FAR *sudist, double  FAR *suangdia);
extern void DLL highmoon(double jd, double FAR *Lambda, double FAR *Beta);
extern double DLL obliqeq(double jd);
extern void DLL ecliptoeq(double jd, double Lambda, double Beta, double FAR *Ra, double FAR *Dec);
extern void DLL definePrecession(double epoch);
extern void DLL precessObject(double ira, double idec, double FAR *ora, double FAR *odec);

//	From SUNCALC.C

extern void ssavetime(HDC hDC, struct tm lt, struct tm ct, RECT tr, COLORREF tcol);
extern void localtimeformat(void);
extern char *rstring(int resid);

//	From SCRNSAVE.C

extern void GetIniEntries(void);
void GetIniSettings(void);
extern int latDeg, latMin, latSec, lonDeg, lonMin, lonSec;
